/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: MyLeadLoader.java,v 1.7 2007/02/05 07:28:55 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.mylead.gui;

import java.net.URI;

import xsul5.MLogger;
import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.gui.Cancelable;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.gui.WaitDialog;
import edu.indiana.extreme.xbaya.mylead.MyLead;
import edu.indiana.extreme.xbaya.mylead.MyLeadException;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class MyLeadLoader implements Cancelable {

    private static final MLogger logger = MLogger.getLogger();

    private XBayaEngine engine;

    private MyLead myLead;

    private Thread loadThread;

    private boolean canceled;

    private WaitDialog loadingDialog;

    /**
     * Constructs a MyLeadWorkflowLoader.
     * 
     * @param engine
     */
    public MyLeadLoader(XBayaEngine engine) {
        this(engine, engine.getMyLead());
    }

    /**
     * Constructs a MyLeadWorkflowLoader.
     * 
     * This method is used to load workflows from a specified location, not from
     * the user's default location.
     * 
     * @param client
     * @param connection
     */
    public MyLeadLoader(XBayaEngine client, MyLead connection) {
        this.engine = client;
        this.myLead = connection;

        this.loadingDialog = new WaitDialog(this, "Loading the Workflow.",
                "Loading the Workflow. " + "Please wait for a moment.",
                this.engine);
    }

    /**
     * @see edu.indiana.extreme.xbaya.gui.Cancelable#cancel()
     */
    public void cancel() {
        this.canceled = true;
        this.loadThread.interrupt();
    }

    /**
     * Loads the workflow.
     * 
     * @param resouceID
     * @param blocking
     *            true for blocking call, false for non-blocking call
     */
    public void load(final String resouceID, boolean blocking) {
        this.canceled = false;

        this.loadThread = new Thread() {
            @Override
            public void run() {
                runInThread(resouceID);
            }
        };
        this.loadThread.start();

        // This has to be the last because it blocks when the dialog is modal.
        this.loadingDialog.show();

        if (blocking) {
            try {
                this.loadThread.join();
            } catch (InterruptedException e) {
                logger.caught(e);
            }
        }
    }

    /**
     * @param resouceID
     */
    private void runInThread(String resouceID) {
        try {
            URI templateID = this.myLead.load(resouceID);
            Workflow workflow = this.engine.getGPELClient().load(templateID);
            this.loadingDialog.hide();
            if (this.canceled) {
                return;
            }
            this.engine.setWorkflow(workflow);
        } catch (MyLeadException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.loadingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.MYLEAD_LOAD_TEMPLATE_ERROR, e);
            }
        } catch (GPELException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.loadingDialog.hide();
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
            }
        } catch (GraphException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.loadingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.GRAPH_FORMAT_ERROR, e);
            }
        } catch (ComponentException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.loadingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.GRAPH_FORMAT_ERROR, e);
            }
        } catch (RuntimeException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.loadingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.MYLEAD_LOAD_TEMPLATE_ERROR, e);
            }
        } catch (Error e) {
            this.loadingDialog.hide();
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
        }
    }

}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
